/*
 * custom.css
 * ~~~~~~~~~
 *
 * Custom stylesheet
 *
 */

div.sphinxsidebar .caption-text {
    font-size: 120%;
}

img.logo {
    width: 100%;
}

.sphinxsidebarwrapper a.current.reference.internal {
    font-weight: bold;
}
